function features_vec = FindWindowHOGs( im )
% calculates HOGs vector for a given image
% we divide every window into block, and every block to cells
% input: im= input image (window of 126x63 size)



nim=im;
nim=double(im);
[r,c]=size(nim);

% Gradients : with the mask [-1,0,1] based on the paper

% X-Gradient
xG=conv2(double(nim),[-1,0,1],'same');

% Y-Gradient
yG=conv2(double(nim(:)),[-1,0,1]','same');
[r,c]=size(nim);
yG=reshape(yG,r,c);

% manually calculate the magnitude and orientaion
% magnitude: 
Gmag=sqrt((xG .^ 2) + (yG .^ 2));
% orientation:
Gdir=atan(yG ./ xG) .* (180 / pi);

% auto gradient from MATLAB (i think it's not good enough)
%[Gmag,Gdir]=imgradient(nim);

% if we have any negative value we flip it by adding 180
negVal=Gdir<0;
Gdir(negVal)=Gdir(negVal)+180;

% create 9 bins: 0-20, 20-40, 40-60, ...., 160-180
bins=floor(Gdir/20)+1;
% in case we had ab over flow
bins(bins > 9) = 9;

windowHeight=126;
windowWidth=63;
% cell size = 6x6 pixels
% block size = 18x18 pixels = 3x3 cells
block_height=18;
block_width=18;
half_block_h=block_height/2;
half_block_w=block_width/2;
% vector hogs counter
v_count=0;

% loop over all blocks in the window with 50% overlapping
% There are 78 blocks of 18x18 pixels each  (50% overlapping)
for i=1 : (windowHeight/half_block_h)-1
    i_start=(i-1)*half_block_h+1;
    for j=1 : (windowWidth/half_block_w)-1
        j_start=(j-1)*half_block_w+1;
        % the orientation of the gradients in current block
        BlockDir=bins(i_start:i_start + 18-1,j_start:j_start + 18-1);
        % the magnitude of the gradients in the current block
        BlockMag=Gmag(i_start:i_start + 18-1,j_start:j_start + 18-1);
        % collect all HOGs in this block
        block_hog_vec=block_HOG(BlockDir,BlockMag);
        % normalize the block hogs to get rid of the effect of lights
        block_vec_norm=block_hog_vec/(sqrt(sum(abs(block_hog_vec).^2)+1^2));
        % hog vector size is (3x3 cells) * (9 bins) = 81
        for k=1:81
          features_vec(v_count+k)=block_vec_norm(k);
        end
        v_count=v_count+81;
    end
end



