function clusters = meanShift(dataPoints,radius)

% apply MeanShift Clustering of 2d data points
%
% Input:
% dataPoints: input data matrix contains locations of the original points
% radius: radius paramter
%
% Output:
% clusters: input data matrix contains locations of clusters


numberOfPoints = size(dataPoints,2);
% count the number of the clusters
numberOfClusters = 0;
% threshold for cheking convergence
threshold = 1e-3*radius;
% returns the clusters we found
clusters = [];
initialPointsNumber = numberOfPoints;
initialPointsIndices = 1:initialPointsNumber;
clusterPower = zeros(1,numberOfPoints,'uint16');
wasVisited = zeros(1,numberOfPoints,'uint8');

% all our points
while initialPointsNumber
    % we choose random point to start our mean with it
    seedPoint = ceil( (initialPointsNumber-1e-6)*rand);
    startMean = initialPointsIndices(seedPoint);
    currentMean = dataPoints(:,startMean);
    clusterMembers = [];
    clusterPower1 = zeros(1,numberOfPoints,'uint16');

    % we loop until we reach a convergence
    while 1
        
        squareDistance = sum((repmat(currentMean, 1, numberOfPoints) - dataPoints).^2);
        pointsInRadius = find(squareDistance < radius^2);
        clusterPower1(pointsInRadius) = clusterPower1(pointsInRadius)+1;
        oldMean = currentMean;
        currentMean = mean(dataPoints(:,pointsInRadius),2);
        clusterMembers   = [clusterMembers pointsInRadius];
        wasVisited(clusterMembers) = 1;
        % if the current mean has no significant change so far, stop 
        if norm(currentMean-oldMean) < threshold
            % look for merging
            mergeWith = 0;
            for newCluster = 1:numberOfClusters
                distToOther = norm(currentMean-clusters(:,newCluster));
                if distToOther < radius/2
                    mergeWith = newCluster;
                    break;
                end
            end
            % if we found a merge, we merge it
            if mergeWith > 0
                clusters(:,mergeWith) = 0.5*(currentMean+clusters(:,mergeWith));
                clusterPower(mergeWith,:) = clusterPower(mergeWith,:) + clusterPower1;
            % else we have no merge --> new cluster
            else
                numberOfClusters = numberOfClusters+1;
                clusters(:,numberOfClusters) = currentMean;
                clusterPower(numberOfClusters,:) = clusterPower1;
            end

            break;
        end
        
    end
    % get all not visited points
    initialPointsIndices = find(wasVisited == 0);
    % get number of activated points
    initialPointsNumber = length(initialPointsIndices);

end
end